@@ -1,13 +1,16 @@
 import collections.abc as cabc
+from copy import copy
 from typing import Union, Optional, Sequence, Any, Mapping, List, Tuple, Callable
 
 import numpy as np
+import pandas as pd
 from anndata import AnnData
 from cycler import Cycler
 from matplotlib.axes import Axes
 from matplotlib.figure import Figure
 from pandas.api.types import is_categorical_dtype
 from matplotlib import pyplot as pl, colors
+from matplotlib.cm import get_cmap
 from matplotlib import rcParams
 from matplotlib import patheffects
 from matplotlib.colors import Colormap
@@ -19,12 +22,14 @@ from .._utils import (
     _FontWeight,
     _FontSize,
     circles,
-    make_projection_available,
+    ColorLike,
+    check_projection,
 )
 from .._docs import (
     doc_adata_color_etc,
     doc_edges_arrows,
     doc_scatter_embedding,
+    doc_scatter_spatial,
     doc_show_save_ax,
 )
 from ... import logging as logg
@@ -59,15 +64,12 @@ def embedding(
     components: Union[str, Sequence[str]] = None,
     layer: Optional[str] = None,
     projection: Literal['2d', '3d'] = '2d',
-    # image parameters
-    img_key: Optional[str] = None,
-    crop_coord: Tuple[int, int, int, int] = None,
-    alpha_img: float = 1.0,
-    bw: bool = False,
-    library_id: str = None,
-    #
+    scale_factor: Optional[float] = None,
     color_map: Union[Colormap, str, None] = None,
+    cmap: Union[Colormap, str, None] = None,
     palette: Union[str, Sequence[str], Cycler, None] = None,
+    na_color: ColorLike = "lightgray",
+    na_in_legend: bool = True,
     size: Union[float, Sequence[float], None] = None,
     frameon: Optional[bool] = None,
     legend_fontsize: Union[int, float, _FontSize, None] = None,
@@ -105,10 +107,22 @@ def embedding(
     -------
     If `show==False` a :class:`~matplotlib.axes.Axes` or a list of it.
     """
-
+    check_projection(projection)
     sanitize_anndata(adata)
+
+    # Setting up color map for continuous values
     if color_map is not None:
-        kwargs['cmap'] = color_map
+        if cmap is not None:
+            raise ValueError("Cannot specify both `color_map` and `cmap`.")
+        else:
+            cmap = color_map
+    cmap = copy(get_cmap(cmap))
+    cmap.set_bad(na_color)
+    kwargs["cmap"] = cmap
+
+    # Prevents warnings during legend creation
+    na_color = colors.to_hex(na_color, keep_alpha=True)
+
     if size is not None:
         kwargs['s'] = size
     if 'edgecolor' not in kwargs:
@@ -121,7 +135,6 @@ def embedding(
         if isinstance(groups, str):
             groups = [groups]
 
-    make_projection_available(projection)
     args_3d = dict(projection='3d') if projection == '3d' else {}
 
     # Deal with Raw
@@ -152,7 +165,7 @@ def embedding(
     # get the points position and the components list
     # (only if components is not None)
     data_points, components_list = _get_data_points(
-        adata, basis, projection, components, img_key, library_id
+        adata, basis, projection, components, scale_factor
     )
 
     # Setup layout.
@@ -200,9 +213,7 @@ def embedding(
 
         if (
             size is not None
-            and isinstance(
-                size, (cabc.Sequence, pandas.core.series.Series, np.ndarray,)
-            )
+            and isinstance(size, (cabc.Sequence, pandas.core.series.Series, np.ndarray))
             and len(size) == adata.shape[0]
         ):
             size = np.array(size, dtype=float)
@@ -225,28 +236,36 @@ def embedding(
     for count, (value_to_plot, component_idx) in enumerate(
         itertools.product(color, idx_components)
     ):
-        color_vector, categorical = _get_color_values(
+        color_source_vector = _get_color_source_vector(
             adata,
             value_to_plot,
             layer=layer,
-            groups=groups,
-            palette=palette,
             use_raw=use_raw,
             gene_symbols=gene_symbols,
+            groups=groups,
+        )
+        color_vector, categorical = _color_vector(
+            adata,
+            value_to_plot,
+            color_source_vector,
+            palette=palette,
+            na_color=na_color,
         )
 
-        # check if higher value points should be plot on top
+        ### Order points
+        order = slice(None)
         if sort_order is True and value_to_plot is not None and categorical is False:
-            order = np.argsort(color_vector)
-            color_vector = color_vector[order]
-            _data_points = data_points[component_idx][order, :]
-
-            # check if 'size' is given (stored in kwargs['s']
-            # and reorder it.
-            if isinstance(size, np.ndarray):
-                size = np.array(size)[order]
-        else:
-            _data_points = data_points[component_idx]
+            # Higher values plotted on top, null values on bottom
+            order = np.argsort(-color_vector, kind="stable")[::-1]
+        elif sort_order and categorical:
+            # Null points go on bottom
+            order = np.argsort(~pd.isnull(color_source_vector), kind="stable")
+        # Set orders
+        if isinstance(size, np.ndarray):
+            size = np.array(size)[order]
+        color_source_vector = color_source_vector[order]
+        color_vector = color_vector[order]
+        _data_points = data_points[component_idx][order, :]
 
         # if plotting multiple panels, get the ax from the grid spec
         # else use the ax value (either user given or created previously)
@@ -290,23 +309,11 @@ def embedding(
                 **kwargs,
             )
         else:
-            if img_key is not None:
-                # had to return size_spot cause spot size is set according
-                # to the image to be plotted
-                img_processed, img_coord, size_spot, cmap_img = _process_image(
-                    adata, data_points, img_key, crop_coord, size, library_id, bw
-                )
-                ax.imshow(img_processed, cmap=cmap_img, alpha=alpha_img)
-                ax.set_xlim(img_coord[0], img_coord[1])
-                ax.set_ylim(img_coord[3], img_coord[2])
-            elif img_key is None and library_id is not None:
-                # order of magnitude similar to public visium
-                size_spot = 70 * size
 
             scatter = (
-                partial(ax.scatter, s=size)
-                if library_id is None
-                else partial(circles, s=size_spot, ax=ax)
+                partial(ax.scatter, s=size, plotnonfinite=True)
+                if scale_factor is None
+                else partial(circles, s=size, ax=ax)  # size in circles is radius
             )
 
             if add_outline:
@@ -355,50 +362,14 @@ def embedding(
                 # if user did not set alpha, set alpha to 0.7
                 kwargs['alpha'] = 0.7 if alpha is None else alpha
 
-            if groups:
-                # first plot non-groups and then plot the
-                # required groups on top
-
-                in_groups = np.array(adata.obs[value_to_plot].isin(groups))
-
-                if isinstance(size, np.ndarray):
-                    in_groups_size = size[in_groups]
-                    not_in_groups_size = size[~in_groups]
-                elif img_key is not None:
-                    in_groups_size = not_in_groups_size = size_spot
-                else:
-                    in_groups_size = not_in_groups_size = size
-
-                # only show grey points if no image is below
-                if library_id is None:
-                    ax.scatter(
-                        _data_points[~in_groups, 0],
-                        _data_points[~in_groups, 1],
-                        s=not_in_groups_size,
-                        marker=".",
-                        c=color_vector[~in_groups],
-                        rasterized=settings._vector_friendly,
-                        **kwargs,
-                    )
-                cax = scatter(
-                    _data_points[in_groups, 0],
-                    _data_points[in_groups, 1],
-                    s=in_groups_size,
-                    marker=".",
-                    c=color_vector[in_groups],
-                    rasterized=settings._vector_friendly,
-                    **kwargs,
-                )
-
-            else:
-                cax = scatter(
-                    _data_points[:, 0],
-                    _data_points[:, 1],
-                    marker=".",
-                    c=color_vector,
-                    rasterized=settings._vector_friendly,
-                    **kwargs,
-                )
+            cax = scatter(
+                _data_points[:, 0],
+                _data_points[:, 1],
+                marker=".",
+                c=color_vector,
+                rasterized=settings._vector_friendly,
+                **kwargs,
+            )
 
         # remove y and x ticks
         ax.set_yticks([])
@@ -412,7 +383,6 @@ def embedding(
             axis_labels = [name + str(x + 1) for x in components_list[component_idx]]
         elif projection == '3d':
             axis_labels = [name + str(x + 1) for x in range(3)]
-
         else:
             axis_labels = [name + str(x + 1) for x in range(2)]
 
@@ -435,25 +405,29 @@ def embedding(
 
         if legend_fontoutline is not None:
             path_effect = [
-                patheffects.withStroke(linewidth=legend_fontoutline, foreground='w',)
+                patheffects.withStroke(linewidth=legend_fontoutline, foreground='w')
             ]
         else:
             path_effect = None
 
-        _add_legend_or_colorbar(
-            adata,
-            ax,
-            cax,
-            categorical,
-            value_to_plot,
-            legend_loc,
-            _data_points,
-            legend_fontweight,
-            legend_fontsize,
-            path_effect,
-            groups,
-            bool(grid),
-        )
+        # Adding legends
+        if categorical:
+            _add_categorical_legend(
+                ax,
+                color_source_vector,
+                palette=_get_palette(adata, value_to_plot),
+                scatter_array=_data_points,
+                legend_loc=legend_loc,
+                legend_fontweight=legend_fontweight,
+                legend_fontsize=legend_fontsize,
+                legend_fontoutline=path_effect,
+                na_color=na_color,
+                na_in_legend=na_in_legend,
+                multi_panel=bool(grid),
+            )
+        else:
+            # TODO: na_in_legend should have some effect here
+            pl.colorbar(cax, ax=ax, pad=0.01, fraction=0.08, aspect=30)
 
     if return_fig is True:
         return fig
@@ -551,7 +525,7 @@ def _get_vmin_vmax(
                         f"Please check the correct format for percentiles."
                     )
                 # interpret value of vmin/vmax as quantile with the following syntax 'p99.9'
-                v_value = np.percentile(color_vector, q=float(v_value[1:]))
+                v_value = np.nanpercentile(color_vector, q=float(v_value[1:]))
             elif callable(v_value):
                 # interpret vmin/vmax as function
                 v_value = v_value(color_vector)
@@ -576,14 +550,30 @@ def _get_vmin_vmax(
 
 
 def _wraps_plot_scatter(wrapper):
-    annots_orig = {
-        k: v for k, v in wrapper.__annotations__.items() if k not in {'adata', 'kwargs'}
+    import inspect
+
+    params = inspect.signature(embedding).parameters.copy()
+    wrapper_sig = inspect.signature(wrapper)
+    wrapper_params = wrapper_sig.parameters.copy()
+
+    params.pop("basis")
+    params.pop("kwargs")
+    wrapper_params.pop("adata")
+
+    params.update(wrapper_params)
+    annotations = {
+        k: v.annotation
+        for k, v in params.items()
+        if v.annotation != inspect.Parameter.empty
     }
-    annots_scatter = {
-        k: v for k, v in embedding.__annotations__.items() if k != 'basis'
-    }
-    wrapper.__annotations__ = {**annots_scatter, **annots_orig}
-    wrapper.__wrapped__ = embedding
+    if wrapper_sig.return_annotation is not inspect.Signature.empty:
+        annotations["return"] = wrapper_sig.return_annotation
+
+    wrapper.__signature__ = inspect.Signature(
+        list(params.values()), return_annotation=wrapper_sig.return_annotation
+    )
+    wrapper.__annotations__ = annotations
+
     return wrapper
 
 
@@ -671,7 +661,7 @@ def diffmap(adata, **kwargs) -> Union[Axes, List[Axes], None]:
     show_save_ax=doc_show_save_ax,
 )
 def draw_graph(
-    adata: AnnData, layout: Optional[_IGraphLayout] = None, **kwargs,
+    adata: AnnData, *, layout: Optional[_IGraphLayout] = None, **kwargs
 ) -> Union[Axes, List[Axes], None]:
     """\
     Scatter plot in graph-drawing basis.
@@ -709,13 +699,24 @@ def draw_graph(
     scatter_bulk=doc_scatter_embedding,
     show_save_ax=doc_show_save_ax,
 )
-def pca(adata, **kwargs) -> Union[Axes, List[Axes], None]:
+def pca(
+    adata,
+    *,
+    annotate_var_explained: bool = False,
+    show: Optional[bool] = None,
+    return_fig: Optional[bool] = None,
+    save: Union[bool, str, None] = None,
+    **kwargs,
+) -> Union[Axes, List[Axes], None]:
     """\
     Scatter plot in PCA coordinates.
 
+    Use the parameter `annotate_var_explained` to annotate the explained variance.
+
     Parameters
     ----------
     {adata_color_etc}
+    annotate_var_explained
     {scatter_bulk}
     {show_save_ax}
 
@@ -723,82 +724,172 @@ def pca(adata, **kwargs) -> Union[Axes, List[Axes], None]:
     -------
     If `show==False` a :class:`~matplotlib.axes.Axes` or a list of it.
     """
-    return embedding(adata, 'pca', **kwargs)
+    if not annotate_var_explained:
+        return embedding(
+            adata, 'pca', show=show, return_fig=return_fig, save=save, **kwargs
+        )
+    else:
+
+        if 'pca' not in adata.obsm.keys() and f"X_pca" not in adata.obsm.keys():
+            raise KeyError(
+                f"Could not find entry in `obsm` for 'pca'.\n"
+                f"Available keys are: {list(adata.obsm.keys())}."
+            )
+
+        label_dict = {
+            'PC{}'.format(i + 1): 'PC{} ({}%)'.format(i + 1, round(v * 100, 2))
+            for i, v in enumerate(adata.uns['pca']['variance_ratio'])
+        }
+
+        if return_fig is True:
+            # edit axis labels in returned figure
+            fig = embedding(adata, 'pca', return_fig=return_fig, **kwargs)
+            for ax in fig.axes:
+                ax.set_xlabel(label_dict[ax.xaxis.get_label().get_text()])
+                ax.set_ylabel(label_dict[ax.yaxis.get_label().get_text()])
+            return fig
+
+        else:
+            # get the axs, edit the labels and apply show and save from user
+            axs = embedding(adata, 'pca', show=False, save=False, **kwargs)
+            if isinstance(axs, list):
+                for ax in axs:
+                    ax.set_xlabel(label_dict[ax.xaxis.get_label().get_text()])
+                    ax.set_ylabel(label_dict[ax.yaxis.get_label().get_text()])
+            else:
+                axs.set_xlabel(label_dict[axs.xaxis.get_label().get_text()])
+                axs.set_ylabel(label_dict[axs.yaxis.get_label().get_text()])
+            _utils.savefig_or_show('pca', show=show, save=save)
+            if show is False:
+                return axs
 
 
 @_wraps_plot_scatter
 @_doc_params(
     adata_color_etc=doc_adata_color_etc,
+    scatter_spatial=doc_scatter_spatial,
     scatter_bulk=doc_scatter_embedding,
     show_save_ax=doc_show_save_ax,
 )
 def spatial(
     adata,
     *,
+    basis: str = "spatial",
+    img: Union[np.ndarray, None] = None,
     img_key: Union[str, None, Empty] = _empty,
     library_id: Union[str, Empty] = _empty,
     crop_coord: Tuple[int, int, int, int] = None,
     alpha_img: float = 1.0,
-    bw: bool = False,
-    size: float = None,
+    bw: Optional[bool] = False,
+    size: float = 1.0,
+    scale_factor: Optional[float] = None,
+    spot_size: Optional[float] = None,
+    na_color: Optional[ColorLike] = None,
+    show: Optional[bool] = None,
+    return_fig: Optional[bool] = None,
+    save: Union[bool, str, None] = None,
     **kwargs,
 ) -> Union[Axes, List[Axes], None]:
     """\
     Scatter plot in spatial coordinates.
 
+    This function allows overlaying data on top of images.
     Use the parameter `img_key` to see the image in the background
     And the parameter `library_id` to select the image.
     By default, `'hires'` and `'lowres'` are attempted.
-    Also by default the first entry of `library_id` is attempted.
+
     Use `crop_coord`, `alpha_img`, and `bw` to control how it is displayed.
     Use `size` to scale the size of the Visium spots plotted on top.
 
+    As this function is designed to for imaging data, there are two key assumptions
+    about how coordinates are handled:
+
+    1. The origin (e.g `(0, 0)`) is at the top left – as is common convention
+    with image data.
+
+    2. Coordinates are in the pixel space of the source image, so an equal
+    aspect ratio is assumed.
+
+    If your anndata object has a `"spatial"` entry in `.uns`, the `img_key`
+    and `library_id` parameters to find values for `img`, `scale_factor`,
+    and `spot_size` arguments. Alternatively, these values be passed directly.
+
     Parameters
     ----------
     {adata_color_etc}
+    {scatter_spatial}
     {scatter_bulk}
     {show_save_ax}
 
     Returns
     -------
     If `show==False` a :class:`~matplotlib.axes.Axes` or a list of it.
+
+    Examples
+    --------
+    This function behaves very similarly to other embedding plots like
+    :func:`~scanpy.pl.umap`
+
+    >>> adata = sc.datasets.visium_sge("Targeted_Visium_Human_Glioblastoma_Pan_Cancer")
+    >>> sc.pp.calculate_qc_metrics(adata, inplace=True)
+    >>> sc.pl.spatial(adata, color="log1p_n_genes_by_counts")
+
+    See Also
+    --------
+    :func:`scanpy.datasets.visium_sge`
+        Example visium data.
+    :tutorial:`spatial/basic-analysis`
+        Tutorial on spatial analysis.
     """
-    if library_id is _empty:
-        library_id = next((i for i in adata.uns['spatial'].keys()))
+    # get default image params if available
+    library_id, spatial_data = _check_spatial_data(adata.uns, library_id)
+    img, img_key = _check_img(spatial_data, img, img_key, bw=bw)
+    spot_size = _check_spot_size(spatial_data, spot_size)
+    scale_factor = _check_scale_factor(
+        spatial_data, img_key=img_key, scale_factor=scale_factor
+    )
+    crop_coord = _check_crop_coord(crop_coord, scale_factor)
+    na_color = _check_na_color(na_color, img=img)
+
+    if bw:
+        cmap_img = "gray"
     else:
-        if library_id not in adata.uns['spatial'].keys():
-            raise KeyError(
-                f"Could not find '{library_id}' in adata.uns['spatial'].keys().\n"
-                f"Available keys are: {list(adata.uns['spatial'].keys())}."
-            )
+        cmap_img = None
+    circle_radius = size * scale_factor * spot_size * 0.5
 
-    spatial_data = adata.uns['spatial'][library_id]
-    if img_key is _empty:
-        img_key = next(
-            (k for k in ['hires', 'lowres'] if k in spatial_data['images']), None,
-        )
-
-    if size is None:
-        size = 1.0
-
-    return embedding(
+    axs = embedding(
         adata,
-        'spatial',
-        img_key=img_key,
-        crop_coord=crop_coord,
-        alpha_img=alpha_img,
-        bw=bw,
-        library_id=library_id,
-        size=size,
+        basis=basis,
+        scale_factor=scale_factor,
+        size=circle_radius,
+        na_color=na_color,
+        show=False,
+        save=False,
         **kwargs,
     )
+    if not isinstance(axs, list):
+        axs = [axs]
+    for ax in axs:
+        cur_coords = np.concatenate([ax.get_xlim(), ax.get_ylim()])
+        if img is not None:
+            ax.imshow(img, cmap=cmap_img, alpha=alpha_img)
+        else:
+            ax.set_aspect("equal")
+            ax.invert_yaxis()
+        if crop_coord is not None:
+            ax.set_xlim(crop_coord[0], crop_coord[1])
+            ax.set_ylim(crop_coord[3], crop_coord[2])
+        else:
+            ax.set_xlim(cur_coords[0], cur_coords[1])
+            ax.set_ylim(cur_coords[3], cur_coords[2])
+    _utils.savefig_or_show('show', show=show, save=save)
+    if show is False or return_fig is True:
+        return axs
 
 
 # Helpers
-
-
 def _get_data_points(
-    adata, basis, projection, components, img_key, library_id
+    adata, basis, projection, components, scale_factor
 ) -> Tuple[List[np.ndarray], List[Tuple[int, int]]]:
     """
     Returns the data points corresponding to the selected basis, projection and/or components.
@@ -902,128 +993,88 @@ def _get_data_points(
         data_points = [np.array(adata.obsm[basis_key])[:, offset : offset + n_dims]]
         components_list = []
 
-    if img_key is not None:
-        spatial_data = adata.uns["spatial"][library_id]
-        if f"tissue_{img_key}_scalef" in spatial_data['scalefactors'].keys():
-            scalef_key = f"tissue_{img_key}_scalef"
-            data_points[0] = np.multiply(
-                data_points[0], spatial_data['scalefactors'][scalef_key],
-            )
-        else:
-            raise KeyError(
-                f"Could not find entry in `adata.uns[spatial][{library_id}]` for '{img_key}'.\n"
-                f"Available keys are: {list(spatial_data['images'].keys())}."
-            )
-    elif img_key is None and basis == "spatial":
-        data_points[0][:, 1] = np.abs(
-            np.subtract(data_points[0][:, 1], np.max(data_points[0][:, 1]))
-        )
+    if scale_factor is not None:  # if basis need scale for img background
+        data_points[0] = np.multiply(data_points[0], scale_factor)
 
     return data_points, components_list
 
 
-def _add_legend_or_colorbar(
-    adata,
+def _add_categorical_legend(
     ax,
-    cax,
-    categorical,
-    value_to_plot,
-    legend_loc,
-    scatter_array,
+    color_source_vector,
+    palette: dict,
+    legend_loc: str,
     legend_fontweight,
     legend_fontsize,
     legend_fontoutline,
-    groups,
     multi_panel,
+    na_color,
+    na_in_legend: bool,
+    scatter_array=None,
+):
+    """Add a legend to the passed Axes."""
+    if na_in_legend and pd.isnull(color_source_vector).any():
+        if "NA" in color_source_vector:
+            raise NotImplementedError(
+                "No fallback for null labels has been defined if NA already in categories."
+            )
+        color_source_vector = color_source_vector.add_categories("NA").fillna("NA")
+        palette = palette.copy()
+        palette["NA"] = na_color
+    cats = color_source_vector.categories
+
+    if multi_panel is True:
+        # Shrink current axis by 10% to fit legend and match
+        # size of plots that are not categorical
+        box = ax.get_position()
+        ax.set_position([box.x0, box.y0, box.width * 0.91, box.height])
+
+    if legend_loc == 'right margin':
+        for label in cats:
+            ax.scatter([], [], c=palette[label], label=label)
+        ax.legend(
+            frameon=False,
+            loc='center left',
+            bbox_to_anchor=(1, 0.5),
+            ncol=(1 if len(cats) <= 14 else 2 if len(cats) <= 30 else 3),
+            fontsize=legend_fontsize,
+        )
+    elif legend_loc == 'on data':
+        # identify centroids to put labels
+        all_pos = (
+            pd.DataFrame(scatter_array, columns=["x", "y"])
+            .groupby(color_source_vector, observed=True)
+            .median()
+        )
+
+        for label, x_pos, y_pos in all_pos.itertuples():
+            ax.text(
+                x_pos,
+                y_pos,
+                label,
+                weight=legend_fontweight,
+                verticalalignment='center',
+                horizontalalignment='center',
+                fontsize=legend_fontsize,
+                path_effects=legend_fontoutline,
+            )
+        # TODO: wtf
+        # this is temporary storage for access by other tools
+        _utils._tmp_cluster_pos = all_pos.values
+
+
+def _get_color_source_vector(
+    adata, value_to_plot, use_raw=False, gene_symbols=None, layer=None, groups=None
 ):
     """
-    Adds a color bar or a legend to the given ax. A legend is added when the
-    data is categorical and a color bar is added when a continuous value was used.
-
-    """
-    # add legends or colorbars
-    if categorical is True:
-        # add legend to figure
-        categories = list(adata.obs[value_to_plot].cat.categories)
-        colors = adata.uns[value_to_plot + '_colors']
-
-        if multi_panel is True:
-            # Shrink current axis by 10% to fit legend and match
-            # size of plots that are not categorical
-            box = ax.get_position()
-            ax.set_position([box.x0, box.y0, box.width * 0.91, box.height])
-
-        if groups is not None:
-            # only label groups with the respective color
-            colors = [colors[categories.index(x)] for x in groups]
-            categories = groups
-
-        if legend_loc == 'right margin':
-            for idx, label in enumerate(categories):
-                color = colors[idx]
-                # use empty scatter to set labels
-                ax.scatter([], [], c=color, label=label)
-            ax.legend(
-                frameon=False,
-                loc='center left',
-                bbox_to_anchor=(1, 0.5),
-                ncol=(
-                    1 if len(categories) <= 14 else 2 if len(categories) <= 30 else 3
-                ),
-                fontsize=legend_fontsize,
-            )
-
-        if legend_loc == 'on data':
-            # identify centroids to put labels
-            all_pos = np.zeros((len(categories), 2))
-            for ilabel, label in enumerate(categories):
-                _scatter = scatter_array[adata.obs[value_to_plot] == label, :]
-                x_pos, y_pos = np.median(_scatter, axis=0)
-
-                ax.text(
-                    x_pos,
-                    y_pos,
-                    label,
-                    weight=legend_fontweight,
-                    verticalalignment='center',
-                    horizontalalignment='center',
-                    fontsize=legend_fontsize,
-                    path_effects=legend_fontoutline,
-                )
-
-                all_pos[ilabel] = [x_pos, y_pos]
-            # this is temporary storage for access by other tools
-            _utils._tmp_cluster_pos = all_pos
-    else:
-        # add colorbar to figure
-        pl.colorbar(cax, ax=ax, pad=0.01, fraction=0.08, aspect=30)
-
-
-def _get_color_values(
-    adata,
-    value_to_plot,
-    groups=None,
-    palette: Union[str, Sequence[str], Cycler, None] = None,
-    use_raw=False,
-    gene_symbols=None,
-    layer=None,
-) -> Tuple[Union[np.ndarray, str], bool]:
-    """
-    Returns the value or color associated to each data point.
-    For categorical data, the return value is list of colors taken
-    from the category palette or from the given `palette` value.
-
-    For non-categorical data, the values are returned
-
-    Returns
-    -------
-    values
-        Values to plot
-    is_categorical
-        Are the values categorical?
+    Get array from adata that colors will be based on.
     """
     if value_to_plot is None:
-        return "lightgray", False
+        # Points will be plotted with `na_color`. Ideally this would work
+        # with the "bad color" in a color map but that throws a warning. Instead
+        # _color_vector handles this.
+        # https://github.com/matplotlib/matplotlib/issues/18294
+        return np.broadcast_to(np.nan, adata.n_obs)
     if (
         gene_symbols is not None
         and value_to_plot not in adata.obs.columns
@@ -1037,35 +1088,54 @@ def _get_color_values(
         values = adata.raw.obs_vector(value_to_plot)
     else:
         values = adata.obs_vector(value_to_plot, layer=layer)
+    if groups and is_categorical_dtype(values):
+        values = values.replace(values.categories.difference(groups), np.nan)
+    return values
 
+
+def _get_palette(adata, values_key: str, palette=None):
+    color_key = f"{values_key}_colors"
+    values = pd.Categorical(adata.obs[values_key])
+    if palette:
+        _utils._set_colors_for_categorical_obs(adata, values_key, palette)
+    elif color_key not in adata.uns or len(adata.uns[color_key]) < len(
+        values.categories
+    ):
+        #  set a default palette in case that no colors or few colors are found
+        _utils._set_default_colors_for_categorical_obs(adata, values_key)
+    else:
+        _utils._validate_palette(adata, values_key)
+    return dict(zip(values.categories, adata.uns[color_key]))
+
+
+def _color_vector(
+    adata, values_key: str, values, palette, na_color="lightgray"
+) -> Tuple[np.ndarray, bool]:
+    """
+    Map array of values to array of hex (plus alpha) codes.
+
+    For categorical data, the return value is list of colors taken
+    from the category palette or from the given `palette` value.
+
+    For continuous values, the input array is returned (may change in future).
+    """
     ###
     # when plotting, the color of the dots is determined for each plot
     # the data is either categorical or continuous and the data could be in
     # 'obs' or in 'var'
+    to_hex = partial(colors.to_hex, keep_alpha=True)
+    if values_key is None:
+        return np.broadcast_to(to_hex(na_color), adata.n_obs), False
     if not is_categorical_dtype(values):
         return values, False
     else:  # is_categorical_dtype(values)
-        color_key = f"{value_to_plot}_colors"
-        if palette:
-            _utils._set_colors_for_categorical_obs(adata, value_to_plot, palette)
-        elif color_key not in adata.uns or len(adata.uns[color_key]) < len(
-            values.categories
-        ):
-            #  set a default palette in case that no colors or few colors are found
-            _utils._set_default_colors_for_categorical_obs(adata, value_to_plot)
-        else:
-            _utils._validate_palette(adata, value_to_plot)
+        color_map = _get_palette(adata, values_key, palette=palette)
+        color_vector = values.map(color_map).map(to_hex)
 
-        color_vector = np.asarray(adata.uns[color_key])[values.codes]
-
-        # Handle groups
-        if groups:
-            color_vector = np.fromiter(
-                map(colors.to_hex, color_vector), '<U15', len(color_vector)
-            )
-            # set color to 'light gray' for all values
-            # that are not in the groups
-            color_vector[~adata.obs[value_to_plot].isin(groups)] = "lightgray"
+        # Set color to 'missing color' for all missing values
+        if color_vector.isna().any():
+            color_vector = color_vector.add_categories([to_hex(na_color)])
+            color_vector = color_vector.fillna(to_hex(na_color))
         return color_vector, True
 
 
@@ -1090,44 +1160,104 @@ def _basis2name(basis):
     return component_name
 
 
-def _process_image(
-    adata, data_points, img_key, crop_coord, scale_spot, library_id, bw=False
-):
-    offset = 100
-    cmap_img = None
-    spatial_data = adata.uns['spatial'][library_id]
-    img = spatial_data['images'][img_key]
-    scalef_key = f"tissue_{img_key}_scalef"
+def _check_spot_size(
+    spatial_data: Optional[Mapping], spot_size: Optional[float]
+) -> float:
+    """
+    Resolve spot_size value.
 
-    # 0.5 needed for optimal matching with spot boundaries
-    # checked with detected_tissue_image.png
-    spot_size = (
-        (
-            spatial_data['scalefactors'][scalef_key]
-            * spatial_data['scalefactors']['spot_diameter_fullres']
-        )
-        * 0.5
-        * scale_spot
-    )
-
-    if crop_coord is not None:
-        crop_coord = np.asarray(crop_coord)
-        if len(crop_coord) != 4:
-            raise ValueError("Invalid crop_coord of length {len(crop_coord)}(!=4)")
-        img_coord = (
-            *crop_coord[:2],
-            *np.ceil(img.shape[0] - crop_coord[2:4]).astype(int),
+    This is a required argument for spatial plots.
+    """
+    if spatial_data is None and spot_size is None:
+        raise ValueError(
+            "When .uns['spatial'][library_id] does not exist, spot_size must be "
+            "provided directly."
         )
+    elif spot_size is None:
+        return spatial_data['scalefactors']['spot_diameter_fullres']
     else:
-        img_coord = [
-            data_points[0][:, 0].min() - offset,
-            data_points[0][:, 0].max() + offset,
-            data_points[0][:, 1].min() - offset,
-            data_points[0][:, 1].max() + offset,
-        ]
+        return spot_size
 
+
+def _check_scale_factor(
+    spatial_data: Optional[Mapping],
+    img_key: Optional[str],
+    scale_factor: Optional[float],
+) -> float:
+    """Resolve scale_factor, defaults to 1."""
+    if scale_factor is not None:
+        return scale_factor
+    elif spatial_data is not None and img_key is not None:
+        return spatial_data['scalefactors'][f"tissue_{img_key}_scalef"]
+    else:
+        return 1.0
+
+
+def _check_spatial_data(
+    uns: Mapping, library_id: Union[Empty, None, str]
+) -> Tuple[Optional[str], Optional[Mapping]]:
+    """
+    Given a mapping, try and extract a library id/ mapping with spatial data.
+
+    Assumes this is `.uns` from how we parse visium data.
+    """
+    spatial_mapping = uns.get("spatial", {})
+    if library_id is _empty:
+        if len(spatial_mapping) > 1:
+            raise ValueError(
+                "Found multiple possible libraries in `.uns['spatial']. Please specify."
+                f" Options are:\n\t{list(spatial_mapping.keys())}"
+            )
+        elif len(spatial_mapping) == 1:
+            library_id = list(spatial_mapping.keys())[0]
+        else:
+            library_id = None
+    if library_id is not None:
+        spatial_data = spatial_mapping[library_id]
+    else:
+        spatial_data = None
+    return library_id, spatial_data
+
+
+def _check_img(
+    spatial_data: Optional[Mapping],
+    img: Optional[np.ndarray],
+    img_key: Union[None, str, Empty],
+    bw: bool = False,
+) -> Tuple[Optional[np.ndarray], Optional[str]]:
+    """
+    Resolve image for spatial plots.
+    """
+    if img is None and spatial_data is not None and img_key is _empty:
+        img_key = next(
+            (k for k in ['hires', 'lowres'] if k in spatial_data['images']),
+        )  # Throws StopIteration Error if keys not present
+    if img is None and spatial_data is not None and img_key is not None:
+        img = spatial_data["images"][img_key]
     if bw:
         img = np.dot(img[..., :3], [0.2989, 0.5870, 0.1140])
-        cmap_img = "gray"
+    return img, img_key
 
-    return img, img_coord, spot_size, cmap_img
+
+def _check_crop_coord(
+    crop_coord: Optional[tuple],
+    scale_factor: float,
+) -> Tuple[float, float, float, float]:
+    """Handle cropping with image or basis."""
+    if crop_coord is None:
+        return None
+    if len(crop_coord) != 4:
+        raise ValueError("Invalid crop_coord of length {len(crop_coord)}(!=4)")
+    crop_coord = tuple(c * scale_factor for c in crop_coord)
+    return crop_coord
+
+
+def _check_na_color(
+    na_color: Optional[ColorLike], *, img: Optional[np.ndarray] = None
+) -> ColorLike:
+    if na_color is None:
+        if img is not None:
+            na_color = (0.0, 0.0, 0.0, 0.0)
+        else:
+            na_color = "lightgray"
+    return na_color
